@@ -140,6 +140,12 @@ public:
 	inline Alignment() : m_len_s(0), m_len_t(0) {
 	}
 
+	inline ~Alignment() {
+		py::gil_scoped_acquire acquire;
+	    m_s_to_t.reset();
+	    m_t_to_s.reset();
+	}
+
 	inline void resize(const size_t len_s, const size_t len_t) {
 		m_len_s = len_s;
 		m_len_t = len_t;
@@ -241,10 +247,10 @@ public:
 typedef core::AlgorithmMetaData Algorithm;
 typedef core::AlgorithmMetaDataRef AlgorithmRef;
 
+template<typename Index>
 class Solution {
 public:
 	typedef float Value;
-	typedef int16_t Index;
 
 	virtual ~Solution() {
 	}
@@ -259,21 +265,24 @@ public:
 	virtual py::object path() const = 0;
 };
 
-typedef std::shared_ptr<Solution> SolutionRef;
+template<typename Index>
+using SolutionRef = std::shared_ptr<Solution<Index>>;
 
+template<typename Index>
 class SolutionIterator {
 public:
 	virtual ~SolutionIterator() {
 	}
 
-	virtual SolutionRef next() = 0;
+	virtual SolutionRef<Index> next() = 0;
 };
 
-typedef std::shared_ptr<SolutionIterator> SolutionIteratorRef;
+template<typename Index>
+using SolutionIteratorRef = std::shared_ptr<SolutionIterator<Index>>;
 
 
 template<typename CellType, typename ProblemType>
-class SolutionImpl : public Solution {
+class SolutionImpl : public Solution<typename CellType::index_type> {
 public:
 	typedef typename CellType::value_type Value;
 	typedef typename CellType::index_type Index;
@@ -343,7 +352,7 @@ public:
 };
 
 template<typename Locality>
-class SolutionIteratorImpl : public SolutionIterator {
+class SolutionIteratorImpl : public SolutionIterator<typename Locality::cell_type::index_type> {
 public:
 	typedef typename Locality::cell_type CellType;
 	typedef typename Locality::problem_type ProblemType;
@@ -365,12 +374,12 @@ public:
 		m_iterator(p_iterator) {
 	}
 
-	virtual SolutionRef next() override {
+	virtual SolutionRef<Index> next() override {
 		const auto r = m_iterator->next();
 		if (r.get()) {
 			return std::make_shared<SolutionImpl<CellType, ProblemType>>(r);
 		} else {
-			return SolutionRef();
+			return SolutionRef<Index>();
 		}
 	}
 };
@@ -681,6 +690,8 @@ public:
 	typedef typename CellType::value_vec_type ValueVec;
 	typedef typename CellType::index_type Index;
 
+	typedef py::gil_scoped_release gil_scoped_release;
+
 private:
 	const std::shared_ptr<Options> m_options;
 	Algorithm m_algorithm;
@@ -692,7 +703,7 @@ private:
 		ValueVec scores;
 
 		{
-			py::gil_scoped_release release;
+		    gil_scoped_release release;
 			p_pairwise.check();
 			m_algorithm.solve(p_pairwise, p_pairwise.batch_len_s(), p_pairwise.batch_len_t());
 			scores = m_algorithm.score(p_pairwise.len_s(), p_pairwise.len_t());
@@ -708,14 +719,14 @@ private:
 		std::array<AlignmentRef<Index>, CellType::batch_size> alignments;
 
 		{
-			py::gil_scoped_release release;
+		    gil_scoped_release release;
 			p_pairwise.check();
 			m_algorithm.solve(p_pairwise, p_pairwise.batch_len_s(), p_pairwise.batch_len_t());
-
-			m_algorithm.template alignment<core::SharedPtrFactory<Alignment<Index>>>(
-				p_pairwise.len_s(), p_pairwise.len_t(), alignments);
 		}
 
+        m_algorithm.template alignment<core::SharedPtrFactory<Alignment<Index>>>(
+            p_pairwise.len_s(), p_pairwise.len_t(), alignments);
+
 		return to_tuple<AlignmentRef<Index>, CellType::batch_size>(alignments);
 	}
 
@@ -726,19 +737,19 @@ private:
 		std::array<AlignmentIteratorRef<Index>, CellType::batch_size> iterators;
 
 		{
-			py::gil_scoped_release release;
+		    gil_scoped_release release;
 			p_pairwise.check();
 			m_algorithm.solve(p_pairwise, p_pairwise.batch_len_s(), p_pairwise.batch_len_t());
-
-			size_t i = 0;
-			for (auto iterator : m_algorithm.template alignment_iterator<
-				core::SharedPtrFactory<Alignment<Index>>>(
-				p_pairwise.len_s(), p_pairwise.len_t(), m_options->remove_dup())) {
-				iterators.at(i++) = std::make_shared<AlignmentIteratorImpl<
-					Index, typename Algorithm::locality_type>>(iterator);
-			}
 		}
 
+        size_t i = 0;
+        for (auto iterator : m_algorithm.template alignment_iterator<
+            core::SharedPtrFactory<Alignment<Index>>>(
+            p_pairwise.len_s(), p_pairwise.len_t(), m_options->remove_dup())) {
+            iterators.at(i++) = std::make_shared<AlignmentIteratorImpl<
+                Index, typename Algorithm::locality_type>>(iterator);
+        }
+
 		return to_tuple<AlignmentIteratorRef<Index>, CellType::batch_size>(iterators);
 	}
 
@@ -746,54 +757,54 @@ private:
 	inline py::tuple _solve_for_solution(
 		const Pairwise &p_pairwise) const {
 
-		std::array<SolutionRef, CellType::batch_size> solutions;
+		std::array<SolutionRef<Index>, CellType::batch_size> solutions;
 
 		{
-			py::gil_scoped_release release;
+		    gil_scoped_release release;
 			p_pairwise.check();
 			m_algorithm.solve(p_pairwise, p_pairwise.batch_len_s(), p_pairwise.batch_len_t());
-
-			std::array<std::shared_ptr<NativeSolution<CellType, ProblemType>>, CellType::batch_size> sol0;
-
-			m_algorithm.template solution<
-				core::SharedPtrFactory<Alignment<Index>>,
-				core::SharedPtrFactory<NativeSolution<CellType, ProblemType>>>(
-					p_pairwise.len_s(),
-					p_pairwise.len_t(),
-					sol0);
-
-			for (int batch_i = 0; batch_i < m_algorithm.batch_size(); batch_i++) {
-				solutions[batch_i] = std::make_shared<SolutionImpl<CellType, ProblemType>>(
-					sol0[batch_i]);
-			}
 		}
 
-		return to_tuple<SolutionRef, CellType::batch_size>(solutions);
+        std::array<std::shared_ptr<NativeSolution<CellType, ProblemType>>, CellType::batch_size> sol0;
+
+        m_algorithm.template solution<
+            core::SharedPtrFactory<Alignment<Index>>,
+            core::SharedPtrFactory<NativeSolution<CellType, ProblemType>>>(
+                p_pairwise.len_s(),
+                p_pairwise.len_t(),
+                sol0);
+
+        for (int batch_i = 0; batch_i < m_algorithm.batch_size(); batch_i++) {
+            solutions[batch_i] = std::make_shared<SolutionImpl<CellType, ProblemType>>(
+                sol0[batch_i]);
+        }
+
+		return to_tuple<SolutionRef<Index>, CellType::batch_size>(solutions);
 	}
 
 	template<typename Pairwise>
 	inline py::tuple _solve_for_solution_iterator(
 		const Pairwise &p_pairwise) const {
 
-		std::array<SolutionIteratorRef, CellType::batch_size> iterators;
+		std::array<SolutionIteratorRef<Index>, CellType::batch_size> iterators;
 
 		{
-			py::gil_scoped_release release;
+		    gil_scoped_release release;
 			p_pairwise.check();
 			m_algorithm.solve(p_pairwise, p_pairwise.batch_len_s(), p_pairwise.batch_len_t());
-
-			size_t i = 0;
-			for (auto iterator : m_algorithm.template solution_iterator<
-				core::SharedPtrFactory<Alignment<Index>>,
-				core::SharedPtrFactory<NativeSolution<CellType, ProblemType>>>(
-				p_pairwise.len_s(), p_pairwise.len_t(), m_options->remove_dup())) {
-
-				iterators.at(i++) = std::make_shared<SolutionIteratorImpl<
-					typename Algorithm::locality_type>>(iterator);
-			}
 		}
 
-		return to_tuple<SolutionIteratorRef, CellType::batch_size>(iterators);
+        size_t i = 0;
+        for (auto iterator : m_algorithm.template solution_iterator<
+            core::SharedPtrFactory<Alignment<Index>>,
+            core::SharedPtrFactory<NativeSolution<CellType, ProblemType>>>(
+            p_pairwise.len_s(), p_pairwise.len_t(), m_options->remove_dup())) {
+
+            iterators.at(i++) = std::make_shared<SolutionIteratorImpl<
+                typename Algorithm::locality_type>>(iterator);
+        }
+
+		return to_tuple<SolutionIteratorRef<Index>, CellType::batch_size>(iterators);
 	}
 
 public:
