@@ -11,20 +11,22 @@ class TestWatermanSmithBeyer(TestCase):
 		# test case is taken from default settings (for logarithmic gap settings) at
 		# http://rna.informatik.uni-freiburg.de/Teaching/index.jsp?toolName=Waterman-Smith-Beyer
 
-		pf = pyalign.problems.general(
+		for pf in self._problems(
+			"ACG",
 			pyalign.problems.Equality(eq=1, ne=-1),
-			direction="maximize")
-		problem = pf.new_problem("CG", "CCGA")
+			direction="maximize"):
 
-		solver = pyalign.solve.GlobalSolver(
-			gap_cost=pyalign.gaps.LogarithmicGapCost(3, 1),
-			codomain=Iterator[pyalign.solve.Alignment])
+			problem = pf.new_problem("CG", "CCGA")
 
-		alignments = list(solver.solve(problem))
+			solver = pyalign.solve.GlobalSolver(
+				gap_cost=pyalign.gaps.LogarithmicGapCost(3, 1),
+				codomain=Iterator[pyalign.solve.Alignment])
 
-		self._check_alignments(
-			alignments,
-			-3.7,
-			[[0, 0], [1, 1]],
-			[[0, 0], [1, 3]],
-			places=1)
+			alignments = list(solver.solve(problem))
+
+			self._check_alignments(
+				alignments,
+				-3.7,
+				[[0, 0], [1, 1]],
+				[[0, 0], [1, 3]],
+				places=1)
